# 짧고굵게배우는 JSP 웹 프로그래밍과 스프링 프레임워크
- 스프링부트 버전 업그레이드에 따른 마이그레이션 방법 입니다.
- STS, 인텔리제이 둘다 가능합니다.

## 스프링 부트 관련 버전 변경 사항
- 스프링 부트 버전: 2.4.4 -> 3.1.3(2023-9월 현재 최신 안정 버전)
- 자바 버전: 11 -> 17 (스프링 부트 3.0 이상 최소 요구 버전)
- 빌드 도구: Maven -> Gradle
- 설정 파일: application.properties -> application.yml
- H2 DB: 1.x -> 2.1.X (2.2는 지원안함)

## 예제소스 import
- gradle 프로젝트이므로 STS, 인텔리 제이 모두 사용 가능.
- 제공된 소스를 사용하는 경우
- 기존 소스에서 혹은 처음부터 프로젝트를 생성하는 경우 아래의 마이그레이션 절차에 따라 진행

### 1. 제공된 소스 -> IntelliJ
  - 학생의 경우 email인증으로 ultimate 무료 사용 가능.
  - https://www.jetbrains.com/community/education/#students
  - jsp_spring.zip 파일의 압축을 해제
  - 인텔리제이에서 프로젝트 오픈
  - application.yml에서 이미지 업로드 경로 확인(PC, MAC 다름)
  - DB tool을 이용해 H2 데이터베이스 생성
  - table.sql 파일을 이용해 H2 데이터베이스에 테이블 생성
  - SpringNewsApplication 실행
  - http://localhost:8080/news/list 로 접속해서 정상 동작 확인  

### 2. 제공된 소스 -> STS
  - jsp_spring.zip 파일의 압축을 해제
  - STS의 경우 import -> Gradle -> Existing Gradle Project 선택
  - H2 console 을 사용해 DB 파일 생성
  - table.sql파일을 이용해 테이블 생성
  - SpringNewsApplication 실행
  - http://localhost:8080/news/list 로 접속해서 정상 동작 확인

### 2. 신규 프로젝트 혹은 기존 소스에서 마이그레이션 절차
> 제공되는 수정 소스를 사용하지 않고 직접 기존 코드를 수정하는 경우

- STS 혹은 인텔리제이(Ultimate) 에서 springInitializr 를 이용해 스프링 부트 프로젝트 생성
- 의존성은: web, h2, jdbc, devtools(옵션 사항)
- 빌드 도구 Gradle-Groovy, 자바 17 버전으로 설정, 스프링 부트 3.1.3 이상.
- 프로젝트 생성후 빌드 도구에 의한 디렉토리 구조, 라이브러리 import 완료 까지 기다임.
- 아래 내용 참조해서 build.gradle 수정후 refresh
- application.yml 파일에 인코딩 설정 추가
- 기존 소스코드를 패키지 디렉토리에 맞게 복사해 올것.
- application.yml 의 첨부파일 경로, NewsDAO.java의 JDBC URL등 변경 사항이 있다면 반영.
- NewsDAO.java 소스코드 SQL 구문에서 PARSEDATETIME 함수를 FORMATDATETIME으로 변경할것.
- NewsWebConfig.java 설정 클래스 추가
- 실행후 정상적으로 동작하는지 확인할것.

## 코드 수정 사항
> 기존 코드 대비 수정 사항 입니다. 

#### build.gradle
- JSTL 관련 의존성 변경됨
- javax.servlet.jstl -> jakarta, org.glassfish 두개 라이브러리로 대체

```gradle
dependencies {
    implementation 'org.springframework.boot:spring-boot-starter-jdbc'
    implementation 'org.springframework.boot:spring-boot-starter-web'
    implementation 'jakarta.servlet.jsp.jstl:jakarta.servlet.jsp.jstl-api'
    implementation 'org.glassfish.web:jakarta.servlet.jsp.jstl'
    implementation 'org.apache.tomcat.embed:tomcat-embed-jasper'
    runtimeOnly 'com.h2database:h2'
    providedRuntime 'org.springframework.boot:spring-boot-starter-tomcat'
    testImplementation 'org.springframework.boot:spring-boot-starter-test'
}
```

#### application.yml
- news.img.dir 은 컴퓨터의 HDD에 이미지 저장 경로, 반드시 뒤에 '/'로 끝나야 함.
- 기본의 경로는 프로젝트 폴더 안쪽에 배치하는 방식이지만 좋지 않습니다. 프로젝트 외부에 저장 권장.
- 윈도우의 경우 기존과 같이 `C:/Dev/springWS/spring_study/src/main/resources/static/img/` 와 같이 하지 마세요..
- `C:/upload/img/` 와 같이 별도 디렉토리를 만들고 경로를 지정.
- yml로 변경되었지만 기존 properties 파일 형식 사용도 가능은 합니다.

```yml
spring:
  mvc:
    view:
      prefix: /WEB-INF/views/
      suffix: .jsp
# UTF-8
server:
  servlet:
    encoding.charset: UTF-8
    encoding.force: true
news:
  img.dir: /Users/dinfree/Downloads/
```

#### NewsDAO.java
- SQL 관련 쿼리중 PARSEDATETIME 함수는 FORMATDATETIME으로 변경됨
- getNews(), getAll()
```java
String sql = "select aid, title, FORMATDATETIME(date,'yyyy-MM-dd hh:mm:ss') as cdate from news";
```

#### NewsWebConfig.java
- 스프링부트에서 프로젝트 외부 경로 접근을 위한 설정
- /img/xx.jpg 와 같은 url 요청을 file:/Users/dinfree/Downloads/xx.jpg 와 같은 경로로 매핑 하게됨.

```java
package com.example.news;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
public class NewsWebConfig implements WebMvcConfigurer {
    @Value("${news.img.dir}")
    private String imgDir;

    @Override
    public void addResourceHandlers(org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry registry) {
        registry.addResourceHandler("/img/**").addResourceLocations("file:" + imgDir);
    }
}
```
#### 특이사항
- 인텔리제이 사용시 DAO 클래스에서 sql 구문에서 DB 객체를 찾을수 없다는 에러가 발생하지만 IDE 기능 관련 사항으로 무시함.